;/*
; * atmega32u2.asm
; *
; *  Created: 30.05.2014 02:30:03
; *   Author: Dominik
; */ 

.set F_MCU		= 16000000
.NOLIST
	;.include "./mcu/m32u2def.inc"
.LIST

.set ENABLE_USB = 1



 .CSEG
 .org $0000
 JMP RESET			; External Pin, Power-on Reset, Brown-out Reset, Watchdog Reset, USB Reset and debugWIRE AVR Reset
 JMP EX_INT0		; External Interrupt Request 0
 JMP EX_INT1		; External Interrupt Request 1
 JMP EX_INT2		; External Interrupt Request 2
 JMP EX_INT3		; External Interrupt Request 3
 JMP EX_INT4		; External Interrupt Request 4
 JMP EX_INT5		; External Interrupt Request 5
 JMP EX_INT6		; External Interrupt Request 6
 JMP EX_INT7		; External Interrupt Request 7
 JMP PC_INT0		; Pin Change Interrupt Request 0
 JMP PC_INT1		; Pin Change Interrupt Request 1
 JMP USB_GEN_INT	; General USB General Interrupt request
 JMP USB_EP_INT		; Endpoint USB Endpoint Interrupt request
 JMP WDT_INT		; Watchdog Time-out Interrupt
 JMP TIMER1_CAPT	; Timer/Counter1 Capture Event
 JMP TIMER1_COMPA	; Timer/Counter1 Compare Match A
 JMP TIMER1_COMPB	; Timer/Counter1 Compare Match B
 JMP TIMER1_COMPC	; Timer/Counter1 Compare Match C
 JMP TIMER1_OVF		; Timer/Counter1 Overflow
 JMP TIMER0_COMPA	; Timer/Counter0 Compare Match A
 JMP TIMER0_COMPB	; Timer/Counter0 Compare match B
 JMP TIMER0_OVF		; Timer/Counter0 Overflow
 JMP SPI_STC		; SPI Serial Transfer Complete
 JMP USART1_RX		; USART1 Rx Complete
 JMP USART1_UDRE	; USART1 Data Register Empty
 JMP USART1_TX		; USART1 Tx Complete
 JMP ANALOG_COMP	; Analog Comparator
 JMP EE_READY		; EEPROM Ready
 JMP SPM_READY		; Store Program Memory Ready


 .NOLIST
.if ENABLE_USB == 1
	.include "./drv/atmega32u2/usb.inc"
.endif
	.include "./drv/atmega32u2/clock.inc"
.LIST

 RESET:			; External Pin, Power-on Reset, Brown-out Reset, Watchdog Reset, USB Reset and debugWIRE AVR Reset
	;Clear Ram
	LDI	XL, LOW(SRAM_START)
	LDI	XH, HIGH(SRAM_START)

	LDI YL, LOW(SRAM_SIZE)
	LDI YH, HIGH(SRAM_SIZE)
	LDI		R16, 0x00
	RAMCLEAN:
		ST X+, R16
		SBIW Y, 1
	BRNE RAMCLEAN
	;Init stack pointer
	LDI		R16, LOW(RAMEND)	;
	LDI		R17, HIGH(RAMEND)	;
	OUT		SPL, R16			;
	OUT		SPH, R17			;
RJMP MAIN

 EX_INT0:		; External Interrupt Request 0
 RETI

 EX_INT1:		; External Interrupt Request 1
 RETI

 EX_INT2:		; External Interrupt Request 2
 RETI

 EX_INT3:		; External Interrupt Request 3
 RETI

 EX_INT4:		; External Interrupt Request 4
 RETI

 EX_INT5:		; External Interrupt Request 5
 RETI

 EX_INT6:		; External Interrupt Request 6
 RETI

 EX_INT7:		; External Interrupt Request 7
 RETI

 PC_INT0:		; Pin Change Interrupt Request 0
 RETI

 PC_INT1:		; Pin Change Interrupt Request 1
 RETI


.if ENABLE_USB == 1
 .INCLUDE "./drv/atmega32u2/usb_interrupts.inc"
.else 
 USB_GEN_INT:	; General USB General Interrupt request

 RETI

 USB_EP_INT:	; Endpoint USB Endpoint Interrupt request

 RETI
.endif

 WDT_INT:		; Watchdog Time-out Interrupt
 RETI

 TIMER1_CAPT:	; Timer/Counter1 Capture Event
 RETI

 TIMER1_COMPA:	; Timer/Counter1 Compare Match A
 RETI
 
 TIMER1_COMPB:	; Timer/Counter1 Compare Match B
 RETI

 TIMER1_COMPC:	; Timer/Counter1 Compare Match C
 RETI

 TIMER1_OVF:	; Timer/Counter1 Overflow
 RETI

 TIMER0_COMPA:	; Timer/Counter0 Compare Match A
 RETI

 TIMER0_COMPB:	; Timer/Counter0 Compare match B
 RETI

 TIMER0_OVF:	; Timer/Counter0 Overflow
 RETI

 SPI_STC:		; SPI Serial Transfer Complete
 RETI

 USART1_RX:		; USART1 Rx Complete
 RETI

 USART1_UDRE:	; USART1 Data Register Empty
 RETI

 USART1_TX:		; USART1 Tx Complete
 RETI

 ANALOG_COMP:	; Analog Comparator
 RETI

 EE_READY:		; EEPROM Ready
 RETI
  
 SPM_READY:		; Store Program Memory Ready
 RETI


 ; Main Init
 MAIN:
	CLI
	SET_CLOCK_DIV CLK_DIV_1
	RCALL INIT_PLL_EXT
	RCALL USB_INIT
	SEI
	RCALL USB_CDC_STATUS_SET_RX
MAIN_END:

 ; Main Loop
 LOOP:
	LDS R16, CDC_STATUS
	ANDI R16, (CDC_RX_bm | CDC_TX_bm)
	CPI R16, 0x00
	BREQ LOOP_TX
	RJMP LOOP_TX_END
	LOOP_TX:
		LDI XL, LOW(CDC_OUT_BUFFER)
		LDI XH, HIGH(CDC_OUT_BUFFER)
		RCALL USB_CDC_TEST_CONTROL_CHAR ;X = RAM_POINTER | RETURN = R0

		LDI R16, CKEY_Backspace
		CP R0, R16
		BREQ LOOP_TX_DEL_CHAR
		RJMP LOOP_TX_DEL_CHAR_END
		LOOP_TX_DEL_CHAR:
			LDI YL, LOW(CDC_DATA)
			LDI YH, HIGH(CDC_DATA)

			LDS XL, CDC_DATA_POSL
			LDS XH, CDC_DATA_POSH

			CLR R16
			CP  XL, R16
			CPC XH, R16
			BREQ LOOP_TX_DEL_CHAR_END

			SBIW X, 1
			STS CDC_DATA_POSL, XL
			STS CDC_DATA_POSH, XH

			ADD YL, XL
			ADC YH, XH

			CLR R16
			ST Y, R16
		RJMP LOOP_TX_FINISH
		LOOP_TX_DEL_CHAR_END:

		; Concat to Data Buffer
		RCALL USB_CDC_CONCAT_TO_DATA
			
		; Echo Input
		LDS R16, CDC_STATUS
		SBRS R16, CDC_ECHO_DISABLE_bp
		RCALL USB_CDC_SEND_ECHO

		RCALL USB_CDC_CMD_TEST_ENTER ; Return = POS = R1:R0 
		CLR R16
		CP  R0, R16
		CPC R1, R16
		BRNE LOOP_CDC_EXEC
		RJMP LOOP_CDC_EXEC_END
		LOOP_CDC_EXEC:
			RCALL USB_CDC_LOAD_CMD ; R1:R0 = CMD_STR_LEN | RETURN = R0 = MAX_ARGS

			; Funktion in usb_cdc_exec_functions.inc
			; Echo CMD
			LDI ZL, LOW(CMD_ECHO)
			LDI ZH, HIGH(CMD_ECHO)
			LDI XL, LOW(CMD_FUNCTION_ECHO)
			LDI XH, HIGH(CMD_FUNCTION_ECHO)
			RCALL USB_CDC_EXEC_CMD ;Z = PGM_CMP_STRING | X = Functions_Pointer | RETURN R0 = [1=True | 0=False]
			
		RJMP LOOP_TX_END
		LOOP_CDC_EXEC_END:
		;-----------------------------------------------
		LOOP_TX_FINISH:
		RCALL USB_CDC_STATUS_SET_TX
	LOOP_TX_END:
 RJMP LOOP
 LOOP_END: