;/*
;* usb_interrupts.inc
;*
;*  Created: 21.02.2016 19:29:58
;*  Author: Dominik Koch
;*/ 

.ifndef USB_INTERRUPTS_INCLUDE
.set USB_INTERRUPTS_INCLUDE = 1

USB_GEN_INT:	; General USB General Interrupt request
PUSH R16
PUSH R17
PUSH R18
PUSH R19
	LDS R16, UDINT
	SBRS R16, EORSTI
	RJMP END_OF_RESET
		;Reset EP0
		CLR R16
		LDI R17, (EP_CONTROL)
		LDI R18, EP0_SIZE
		CLR R19
		RCALL USB_INIT_EP ;R16 EP_NUM, R17 EP_TYPE + EP_DIR, R18 EP_SIZE, R19 EP_BANKS

		LDS R16, UEIENX
		SBR	R16, (1<<RXSTPE)
		STS UEIENX, R16

		LDS R16, UDINT
		CBR	R16, (1<<EORSTI)
		STS UDINT, R16
	END_OF_RESET:
POP R19
POP R18
POP R17
POP R16
RETI

USB_EP_INT:	; Endpoint USB Endpoint Interrupt request
PUSH R16
IN R16, SREG
PUSH R16
PUSH R17
PUSH R18
PUSH R19
PUSH XL
PUSH XH
	LDS R17, UEINT

	SBRS R17, 0	; when EP0 Selected
	RJMP USB_EP_INT_EP0_END
	USB_EP_INT_EP0:
		LDI R16, 0
		STS UENUM, R16
		RCALL USB_EP0_CONTROL
	USB_EP_INT_EP0_END:
	
	;EP_CDC_C_IN
	SBRS R17, 1	; when EP1 Selected
	RJMP USB_EP_INT_EP1_END	 
	USB_EP_INT_EP1:
		LDI R16, 1
		STS UENUM, R16

		LDS R16, UEINTX
		
		;CBR R16, (1<<TXINI)
		;STS UEINTX, R16

		;CBR R16, (1<<FIFOCON)
		;STS UEINTX, R16

		CBR R16, (1<<NAKINI)
		STS UEINTX, R16
		RJMP USB_EP_INT_END
	USB_EP_INT_EP1_END:

	;EP_CDC_D_OUT
	SBRS R17, 2	; when EP2 Selected
	RJMP USB_EP_INT_EP2_END
	USB_EP_INT_EP2:
		LDI R16, 2
		STS UENUM, R16

		LDS R18, CDC_STATUS
		SBRC R18, CDC_RX_bp
		RJMP USB_EP_INT_EP2_REC
		RJMP USB_EP_INT_EP2_NAK

		USB_EP_INT_EP2_REC:
			LDS R16, UEINTX
			CBR R16, (1<<RXOUTI)
			STS UEINTX, R16
			; Copy USB Data to Ram Pointer
			LDI XL, LOW(CDC_OUT_BUFFER)
			LDI XH, HIGH(CDC_OUT_BUFFER)
			RCALL USB_DATA_TO_RAM ; X = RAM_Pointer | Return = R0 = Byte Counter
			; Save String Length
			STS CDC_OUT_BUFFER_COUNT, R0
			; Zero Terminated String
			CLR R16
			ST X, R16

			RCALL USB_CDC_STATUS_CLEAR_RX
			
			CBR R16, (1<<FIFOCON)
			STS UEINTX, R16
			RJMP USB_EP_INT_END
		USB_EP_INT_EP2_REC_END:

		USB_EP_INT_EP2_NAK:
			CBR R16, (1<<NAKINI)
			STS UEINTX, R16
			RJMP USB_EP_INT_END
		USB_EP_INT_EP2_NAK_END:
	USB_EP_INT_EP2_END:

	;EP_CDC_D_IN
	SBRS R17, 3	; when EP3 Selected
	RJMP USB_EP_INT_EP3_END
	USB_EP_INT_EP3:
		LDI R16, 3
		STS UENUM, R16

		LDS R18, CDC_STATUS
		SBRC R18, CDC_TX_bp
		RJMP USB_EP_INT_EP3_SEND
		RJMP USB_EP_INT_EP3_NAK

		USB_EP_INT_EP3_SEND:
			LDS R16, UEINTX
			CBR R16, (1<<TXINI)
			STS UEINTX, R16
		
			LDS R19, CDC_OUT_BUFFER_COUNT
			LDI XL, LOW(CDC_OUT_BUFFER)
			LDI XH, HIGH(CDC_OUT_BUFFER)
			CPI R19, 0x00
			BREQ USB_EP_INT_EP3_OUT_LOOP_END

			USB_EP_INT_EP3_OUT_LOOP:
				LD R18, X+
				STS UEDATX, R18
				DEC R19
			BRNE USB_EP_INT_EP3_OUT_LOOP
			CLR R19
			STS CDC_OUT_BUFFER_COUNT, R19
			USB_EP_INT_EP3_OUT_LOOP_END:

			RCALL USB_CDC_STATUS_CLREAR_TX
			RCALL USB_CDC_STATUS_SET_RX

			CBR R16, (1<<FIFOCON)
			STS UEINTX, R16
			RJMP USB_EP_INT_END
			;RJMP USB_EP_INT_EP3_END
		USB_EP_INT_EP3_SEND_END:

		USB_EP_INT_EP3_NAK:
			CBR R16, (1<<NAKINI)
			STS UEINTX, R16
			RJMP USB_EP_INT_END
		USB_EP_INT_EP3_NAK_END:
	USB_EP_INT_EP3_END:

	; Not in use
	SBRS R17, 4	; when EP4 Selected
	RJMP USB_EP_INT_EP4_END
	USB_EP_INT_EP4:
		LDI R16, 4
		STS UENUM, R16

		LDS R16, UEINTX
		;CBR R16, (1<<TXINI)
		;STS UEINTX, R16
		;CBR R16, (1<<FIFOCON)
		;STS UEINTX, R16
		CBR R16, (1<<NAKINI)
		STS UEINTX, R16
		RJMP USB_EP_INT_END
	USB_EP_INT_EP4_END:
USB_EP_INT_END:
POP XH
POP XL
POP R19
POP R18
POP R17
POP R16
OUT SREG, R16
POP R16
RETI
.ENDIF
